#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include "onie_tlvinfo.h"

static u_int8_t eeprom[SYS_EEPROM_SIZE];
/*
 *  This macro defines the sys_eeprom command line command.
 */
void cmd_usage()
{
    static const char *usage =
	"Display and program the system EEPROM data block.\n"
	"Usage: sys-eeprom [-h][-l] [-e] [-s <code>=<value>,...]\n"
	"   With no arguments display the EEPROM contents.\n"
	"   -h --help\n"
	"      Display usage\n"
	"   -l --list\n"
	"      List the understood TLV codes and names.\n"
	"   -e --erase\n"
	"      Reset the EEPROM data.\n"
	"   -g --get <code>\n"
	"      Look up a TLV by code and write the value to stdout.\n"
	"   -s --set <code>=<value>,<code>=<value>...\n"
	"      Set a TLV code to a value.\n"
	"      If no value, TLV is deleted.\n";

    fprintf(stderr, "%s", usage);
    exit(1);
}
/*
 *  do_sys_eeprom
 *  This function implements the sys_eeprom command.
 */
int main(int argc, char * const argv[])
{
    int count = 0;
    int err = 0;
    int update = 0;
    char *value, *subopts, *tname;
    int index, c, option_index, tcode;
    size_t i;
    char tlv_value[TLV_DECODE_VALUE_MAX_LEN];

    const size_t tlv_code_count = sizeof(tlv_code_list) /
	sizeof(tlv_code_list[0]);

    char *tokens[tlv_code_count + 1];
    const char *short_options = "hels:g:";
    const struct option long_options[] = {
	{"help",    no_argument,          0,    'h'},
	{"list",    no_argument,          0,    'l'},
	{"erase",   no_argument,          0,    'e'},
	{"set",     required_argument,    0,    's'},
	{"get",     required_argument,    0,    'g'},
	{0,         0,                    0,      0},
    };

    for (i = 0; i < tlv_code_count; i++) {
	    tokens[i] = (char *) malloc(6);
	    sprintf(tokens[i], "0x%x", tlv_code_list[i].m_code);
    }
    tokens[tlv_code_count] = NULL;

    while (TRUE) {
	c = getopt_long(argc, argv, short_options,
			long_options, &option_index);
	if (c == EOF)
	    break;

	count++;
	switch (c) {
	case 'h':
	    cmd_usage();
	    break;

	case 'l':
	    show_tlv_code_list();
	    break;

	case 'e':
	    if (read_eeprom(eeprom)) {
                err = 1;
		goto syseeprom_err;
	    }
	    update_eeprom_header(eeprom);
	    update = 1;
	    break;

	case 's':
	    subopts = optarg;
	    while (*subopts != '\0' && !err) {
		if ((index = getsubopt(&subopts, tokens, &value)) != -1) {
		    if (read_eeprom(eeprom)) {
                        err = 1;
			goto syseeprom_err;
		    }
		    tcode = strtoul(tokens[index], NULL, 0);
		    for (i = 0; i < tlv_code_count; i++) {
			if (tlv_code_list[i].m_code == tcode) {
			    tname = tlv_code_list[i].m_name;
			}
		    }
		    if (tlvinfo_delete_tlv(eeprom, tcode) == TRUE) {
			    printf("Deleting TLV 0x%x: %s\n", tcode, tname);
		    }
		    if (value) {
			if (!tlvinfo_add_tlv(eeprom, tcode, value)) {
                            err = 1;
			    goto syseeprom_err;
			} else {
			    printf("Adding   TLV 0x%x: %s\n", tcode, tname);
			}
		    }
		    update = 1;
		} else {
		    err = 1;
		    printf("ERROR: Invalid option: %s\n", value);
		    goto syseeprom_err;
		}
	    }
	break;

	case 'g':
            if (read_eeprom(eeprom)) {
                err = 1;
                goto syseeprom_err;
            }
            tcode = strtoul(optarg, NULL, 0);
            if (tlvinfo_decode_tlv(eeprom, tcode, tlv_value)) {
                printf("%s\n", tlv_value);
            } else {
                err = 1;
                printf("ERROR: TLV code not present in EEPROM: 0x%02x\n", tcode);
            }
            goto syseeprom_err;
	break;

	default:
	    cmd_usage();
            err = 1;
	    break;
	}
    }
    if (!count) {
	if (argc > 1) {
	    cmd_usage();
            err = 1;
	} else {
	    if (read_eeprom(eeprom)) {
                err = 1;
                goto syseeprom_err;
            }
	    show_eeprom(eeprom);
	}
    }
    if (update) {
	if (prog_eeprom(eeprom)) {
            err = 1;
            goto syseeprom_err;
        }
	show_eeprom(eeprom);
    }
syseeprom_err:
    for (i = 0; i < tlv_code_count; i++) {
	free(tokens[i]);
    }
    return  (err == 0) ? 0 : 1;
}
